(require (lib "list.ss"))

(load "ps1.scm")

;; Abstractions for working with nodes.

;; make-node: you can create an AND node of several statements with
;;   (make-node 'AND statements). It's really just cons, but it's an abstraction.
(define make-node cons)

;; node-type gives you back AND if it's an AND node, and OR if it's an OR node.
(define node-type car)

;; node-parts gives you a list of the subgoals (branches) of a node.
(define node-parts cdr)

;; Test if an expression is an AND node.
(define (AND-node? node)
  (and (pair? node) (eq? (car node) 'AND)))

;; Test if an expression is an OR node.
(define (OR-node? node)
  (and (pair? node) (eq? (car node) 'OR)))

;; Define some basic nodes. The empty AND represents unconditional success.
;; The empty OR represents unconditional failure.
(define empty-AND '(AND))
(define empty-OR '(OR))
(define *succeed* empty-AND)
(define *fail* empty-OR)

;; You need to write this function.
;; It takes in a goal tree and simplifies it, according to rules described
;; in the problem set.
;;
;; Example: (simplify '(AND (AND 3 4) (OR 5 6 (OR))))  => (AND 3 4 (OR 5 6))
(define (simplify node)
  (cond
    ((unit? node) node)
    ((AND-node? node)
      (if (and (pair? (cdr node)) (null? (cdr (cdr node))))
          (simplify (car (cdr node)))
      (letrec ((replace-if-fail (lambda (tree)  
              ;This checks for universal failure and replaces the entire tree, if found
              (if (member *fail* (node-parts tree)) *fail* tree))))     
      (replace-if-fail      
      (cons 'AND ;The tree starts off with an AND node
                  
        (letrec ((walk (lambda (list)
                         
             (cond 
               ((not (pair? list)) list)  ;BASE CASE --> We've reached the end  
               ;ELSE recursive cases
               ((not (pair? (car list)))
                     (cons (car list) (walk (cdr list))))  ;Symbol leaf, add back and continue
               ;Therefore the car must be a pair (either ('AND ...) or ('OR ...)
               ((eqv? (car (car list)) 'AND)
                     ;2 Cases to consider                  
                     (cond 
                        ;If the AND has no elements (i.e. it is a *succeed*, then we can ignore it)
                        ((equal? *succeed* (car list)) (walk (cdr list))) 
                        ;If there is only one element, just put that
                        ;((not (pair? (cdr (car list)))) (cons (cdr (car list)) (walk (cdr list))))                             
                        ;Otherwise, append the two nodes and walk them both
                        (else (walk (append (cdr (car list)) (cdr list))))
                     )
               )
               ;The pair is ('OR ...)
               (else (cons (simplify (car list)) (walk (cdr list))))
             )
           )))
         ;After the 'AND symbol, walk the rest of the tree recursively 
         (walk (node-parts node)))))
        )
      )
     ) 
    ((OR-node? node)
     (if (and (pair? (cdr node)) (null? (cdr (cdr node))))
          (simplify (car (cdr node)))     
      (letrec ((replace-if-succ (lambda (tree)  
              ;This checks for universal success and replaces the entire tree, if found
              (if (member *succeed* (node-parts tree)) *succeed* tree))))     
      (replace-if-succ      
      (cons 'OR ;The tree starts off with an OR node
                  
        (letrec ((walk (lambda (list)
                         
             (cond 
               ((not (pair? list)) list)  ;BASE CASE --> We've reached the end  
               ;ELSE recursive cases
               ((not (pair? (car list)))
                     (cons (car list) (walk (cdr list))))  ;Symbol leaf, add back and continue
               ;Therefore the car must be a pair (either ('AND ...) or ('OR ...)
               ((eqv? (car (car list)) 'OR)
                     ;2 Cases to consider                 
                     (cond 
                        ;If the OR has no elements (i.e. it is a *fail*, then we can ignore it)
                        ((equal? *fail* (car list)) (walk (cdr list)))                            
                        ;Otherwise, append the two nodes and walk them both
                        (else (walk (append (cdr (car list)) (cdr list))))
                     )
               )
               ;The pair is ('AND ...)
               (else (cons (simplify (car list)) (walk (cdr list))))
             )
           )))
         ;After the 'AND symbol, walk the rest of the tree recursively 
         (walk (node-parts node)))))
        )
      )
    )
  )
)

;Returns true if the items is atomic, or a list which doesn't have 'AND or 'OR as a prefix
(define (unit? node) (if (atom? node) #t (if (or (AND-node? node) (OR-node? node)) #f #t)))


;;TEST!
;(simplify '(AND (OR (OR (opus has feathers)) (AND (OR (opus flies)) (OR (opus lays eggs)))) (OR (opus does not fly)) (OR (opus swims)) (OR (opus has black and white color))))

;(AND (OR (OR (opus has feathers)) (AND (OR (opus flies)) (OR (opus lays eggs)))) (OR (opus does not fly)) (OR (opus swims)) (OR (opus has black and white color)))