(require (lib "list.ss"))
(require (lib "pretty.ss"))
(load "search.scm")

;Your answers for the true and false questions will be in the following form.  
; Note that the tester will accept either answer as correct.  The hidden tester will not.
; Your answers will look like one of the two below:
;(define answer-1.5.0 #t)
;(define answer-1.5.0 #f)

; 1.5.0: True or false - Hill Climbing search is guaranteed to find a solution if there is a solution
(define answer-1.5.0 #f)

; 1.5.1: True or false - Best-first search will give an optimal search result (shortest path length).
(define answer-1.5.1 #t)

; 1.5.2: True or false - Best-first search and hill climbing make use of heuristic values of nodes.
(define answer-1.5.2 #t)

; 1.5.3: True or false - A* uses an extended list.
(define answer-1.5.3 #t)

; 1.5.4: True or false - Breadth first search is guaranteed to return a path with the shortest number of nodes.
(define answer-1.5.4 #t)

; 1.5.5: True or false - Best-first search is a refinement of the breadth-first search method.
(define answer-1.5.5 #t)

; 1.5.6: True or false - The regular branch and bound uses heuristic values to speed up the search for an optimal path.
(define answer-1.5.6 #t)

;; Problem 2.1
(define (bfs nodes edges useExtendedList? extended queue start goal)
  (if (not (null? queue))
      (if (and useExtendedList? (member (get-last (car queue)) extended))
          ;If we're using the extended list, check to make sure our next path is valid
          (bfs nodes edges useExtendedList? extended (cdr queue) start goal)
          
          (if (equal? (get-last (car queue)) goal)
              ;We've found the node, return the path!
              (create-search-solution (append extended (list goal)) (car queue))
              ;Otherwise, expand that node and add expanded paths to the queue
              (bfs nodes
                   edges
                   useExtendedList?
                   (append extended (list (get-last (car queue))))
                   (update-queue queue edges nodes "BFS")
                   start
                   goal
                   )
              )
          )
      "BFS failed to find a path!"
  )
)


(define (update-queue queue edges nodes type)
  (let ((extensions (exclude (get-connected-nodes (get-last (car queue)) edges) (car queue))))
    (if (not (null? extensions))
        ;If there are nodes to extend, add them to the queue
        (if (null? (cdr queue))
            (map (lambda (end) (append (car queue) (list end))) (if (string=? type "HC") (sort-by-heuristic extensions nodes) extensions))
            (cond
                ((string=? type "BFS") (append (cdr queue) (map (lambda (end) (append (car queue) (list end))) extensions)))
                ((string=? type "DFS") (append (map (lambda (end) (append (car queue) (list end))) extensions) (cdr queue)))
                (else (append (map (lambda (end) (append (car queue) (list end))) (sort-by-heuristic extensions nodes)) (cdr queue)))
            )
        )
        ;Otherwise, return the queue with the nodes popped
        (cdr queue)
    )
  )
) 

;; Once you have completed the breadth-first search, this part should be very simple to complete.
(define (dfs nodes edges useExtendedList? extended queue start goal)
  (if (not (null? queue))
      (if (and useExtendedList? (member (get-last (car queue)) extended))
          ;If we're using the extended list, check to make sure our next path is valid
          (dfs nodes edges useExtendedList? extended (cdr queue) start goal)
          
          (if (equal? (get-last (car queue)) goal)
              ;We've found the node, return the path!
              (create-search-solution (append extended (list goal)) (car queue))
              ;Otherwise, expand that node and add expanded paths to the queue
              (dfs nodes
                   edges
                   useExtendedList?
                   (append extended (list (get-last (car queue))))
                   (update-queue queue edges nodes "DFS")
                   start
                   goal
                   )
              )
          )
      "DFS failed to find a path!"
  )
)

;;This function takes two lists and returns the first list minus any elements that also exist in the second list
(define (exclude list1 list2)
  (if (pair? list1)
      (if (not (member (car list1) list2)) 
          (cons (car list1) (exclude (cdr list1) list2))
          (exclude (cdr list1) list2))
      '()
  )        
)

;;This function extends a path, taking the last node in the path, finding all its connections and returning this as a list
;;If useExtendedList? is #t then it will ignore any extensions already in extended
(define (extend-path edges path useExtendedList? extended)
  (if (eqv? useExtendedList? #t)
      (exclude (get-connected-nodes (get-last path) edges) extended)
      (get-connected-nodes (get-last path) edges)
  )
)
   
;; Now we're going to add some heuristics into the search.  Remember that hill-climbing is a modified version of depth-first search.  You might find the function sort-by-heuristic in search.scm very useful.
(define (hill-climbing nodes edges useExtendedList? extended queue start goal)
  (if (not (null? queue))
      (if (and useExtendedList? (member (get-last (car queue)) extended))
          ;If we're using the extended list, check to make sure our next path is valid
          (hill-climbing nodes edges useExtendedList? extended (cdr queue) start goal)
          
          (if (equal? (get-last (car queue)) goal)
              ;We've found the node, return the path!
              (create-search-solution (append extended (list goal)) (car queue))
              ;Otherwise, expand that node and add expanded paths to the queue
              (hill-climbing
                   nodes
                   edges
                   useExtendedList?
                   (append extended (list (get-last (car queue))))
                   (update-queue queue edges nodes "HC")
                   start
                   goal
                   )
              )
          )
      "HC failed to find a path!"
  )  
)


;; Now we're going to try optimal search.  The previous searches haven't used edge distances in the calculation.  In order to compute the optimal path, complete the helper function below.  
;; The function you are complete is below.  It takes in a list of node-ids and returns the distance of edges along the path.  If a path does not exist, it returns #f.
(define (path-length node-ids edges)
  (if (null? (cdr node-ids)) 
      0
      (+ (edge-distance (get-edge (car node-ids) (cadr node-ids) edges)) (path-length (cdr node-ids) edges))
  )
)

(define (branch-and-bound nodes edges useExtendedList? extended queue start goal)
  (create-search-solution extended '()))

(define (a-star nodes edges extended queue start goal)
  (if (not (null? queue))
      (if (member (get-last (car queue)) extended)
          ;If we're using the extended list, check to make sure our next path is valid
          (a-star nodes edges extended (cdr queue) start goal)
          
          (if (equal? (get-last (car queue)) goal)
              ;We've found the node, return the path!
              (create-search-solution (append extended (list goal)) (car queue))
              ;Otherwise, expand that node and add expanded paths to the queue
              (a-star
                   nodes
                   edges
                   (append extended (list (get-last (car queue))))
                   (sort-paths-by-estimate (update-queue queue edges nodes "BFS") nodes edges)
                   start
                   goal
                   )
              )
          )
      "A* failed to find a path!"
  )
  
)

(define (sort-paths-by-estimate paths nodes edges)
 (sort paths (lambda (p1 p2) (< (+ (path-length p1 edges) (get-heuristic (get-last p1) nodes))
                                (+ (path-length p2 edges) (get-heuristic (get-last p2) nodes)))))
)


;; Its useful to determine if a graph has consistent and admissible heuristics.  You've seen graphs with heuristics that are admissible, but not consistent.  Have you seen any graphs that are consistent, but not admissible?  This should give you a clue that helps simply the computation.
(define (isAdmissibleAndConsistent? graph) 
  ;If a graph's edges are consistent, it must be admissible, since consistent edge lengths (where |h(n1)-h(n2)| < |e1|) means no heuristic can overestimate the length to the goal
  (is-consistent? (get-nodes graph) (get-edges graph))
)

(define (is-consistent? nodes edges)
 (cond
  ((null? edges) #t)
  ((< (edge-distance (car edges)) (abs (- (get-heuristic (car (edge-nodes (car edges))) nodes) (get-heuristic (car (cdr (edge-nodes (car edges)))) nodes)))) #f)
  (else (is-consistent? nodes (cdr edges))) 
 )
)

;(isAdmissibleAndConsistent? thegraph)

(define how-many-hours-this-pset-took "-")
(define what-i-found-interesting "-")
(define what-i-found-boring "-")
