;;; Sudoku
(load "sudoku.scm")
(load "constraint-graphics.scm")
(load "8pawns.scm")

;;1.2

;;can a cell propagate constraint onto other cells?
(define (can-propagate? cell)
  (if (null? (cdr (cell-values cell))) ;Is there only one value?
      #t
      #f
  )
)

;;propagate one cell onto others.  others are all
;; cells which cannot share a number with cell.
;;return true if any cells modified, false otherwise.
(define (propagate-one-cell cell others)
  (if (can-propagate? cell) 
    (or 
     (if (null? others) 
       #f
       (if (member (car (cell-values cell)) (cell-values (car others)))
           (or (modify-cell cell others) (propagate-one-cell cell (cdr others)))
           (propagate-one-cell cell (cdr others))
       )
     )
    )     
    #f
  )
)

;Modifies a cell to remove the constraint values and returns true (note, must check whether anything *can* be removed first!
(define (modify-cell cell others) (set-values! (car others) (remove (car (cell-values cell)) (cell-values (car others)))) #t)

;;1.3

;; go through all cells in the board once and apply the 
;;propagate-one-cell procedure for the right "others"
;;values.  return true if any cells modified, false otherwise
(define (propagate-one-round board)
 (let loop ((n 0) (modified? #f))
   (if (> n 80) 
       modified?
       (loop (+ n 1) (or (propagate-one-cell (get-cell-by-number board n) (cells-in-same-row (get-cell-by-number board n) board))
                         (propagate-one-cell (get-cell-by-number board n) (cells-in-same-col (get-cell-by-number board n) board))
                         (propagate-one-cell (get-cell-by-number board n) (cells-in-same-square (get-cell-by-number board n) board))
                          modified?))
   )
 )
)

;; constraint-propagate as long as it makes a difference
;;return the number of times through the board.
(define (constraint-propagate board count)
 (if (propagate-one-round board) (constraint-propagate board (+ count 1)) count)
)

;;; Multiple choice

; 2.1.1. Two computerized players are playing a game. Player MM does minimax search
; to depth 6 to decide on a move. Player AB does alpha-beta search to depth 6.
; The game is played without a time limit. Which player will play better?
;
;   1. MM will play better than AB.
;   2. AB will play better than MM.
;   3. They will play with the same level of skill.
(define answer-2.1.1 3)

; 2.1.2. Two computerized players are playing a game with a time limit. Player MM
; does minimax search with iterative deepening, and player AB does alpha-beta search
; with iterative deepening. Each one returns a result after it has used 1/3 of its
; remaining time. Which player will play better?
;
;   1. MM will play better than AB.
;   2. AB will play better than MM.
;   3. They will play with the same level of skill.
(define answer-2.1.2 2)

;;; 2.2. The eight-pawn game

;; This section will contain occasional lines that you can uncomment to play
;; the eight-pawn game interactively. Be sure to re-comment them when you're done
;; with them. *Please* don't turn in a problem set that sits there asking GraderBot
;; to play a game!
;; 
;; Uncomment this line to play a game as white:
;(play-game startpos (human-player) (chatty-minimax-player 5 simple-evaluate))

;; Uncomment this line to play a game as black:
;(play-game startpos (human-player) (chatty-minimax-player 5 simple-evaluate))

;; Or watch the computer play against itself:
;(play-game startpos (chatty-minimax-player 5 simple-evaluate) (chatty-minimax-player 5 simple-evaluate))

;; Write a new evaluation function that tries to win as soon as possible,
;; or lose as late as possible, when it decides that one side is certain to win.
;; You don't have to change how it evaluates non-winning positions.
;; Look at simple-evaluate in 8pawns.scm for an example.
(define (focused-evaluate pos player depth)
  (let ((h (simple-evaluate pos player depth)))
    (cond 
     ((= 1000 h) (+ 1100 depth)) ;If we know we have a certain win, favor shallower positions (depth indicates how far *to go* in search!)
     ((= -1000 h) (- -1100 depth)) ;If we know we're going to lose, favor deeper positions  
     (else h)
    )
  )   
) 

;; You can try out your new evaluation function by uncommenting this line:
;(play-game startpos (human-player) (chatty-minimax-player 5 focused-evaluate))

;; Write an alpha-beta-search procedure that acts like the minimax-search procedure,
;; but uses alpha-beta pruning to avoid searching bad ideas that can't improve
;; the result. The tester will check your pruning by counting the number of
;; static evaluations you make.
;;
;; You can use minimax-search in 8pawns.scm as an example.
(define (alpha-beta-search position depth player max? evaluation-function alpha beta)
 (if (or (= depth 0) (check-loss position))
      ; Stop expanding the game tree here, either because someone won or
      ; we've expanded far enough.
      ; We're not making any moves from here, so return #f as the move.
      (list #f (evaluation-function position player depth))
      
      ; Otherwise, find the valid moves, recursively evaluate them,
      ; and return the one with the highest/lowest score.
      
      (foldl (if max? maximize-position minimize-position)
             (list #f (if max? -2000 2000)) ; start with an extreme value that won't actually be returned
             
             ; Expand every position we can get to using minimax-search, and attach to each one 
             ; the position we move to so that we know which move to make.
             ;
             ; The cadr of the result obtained from minimax-search is the heuristic value.
             ; We disregard the car, which is the move to make from there - we only need
             ; to make one move at a time.
             (letrec ((examine (lambda (moves new-alpha new-beta)
                            (let ((move-eval (list (car moves) (cadr (alpha-beta-search (car moves) (- depth 1) player (not max?) evaluation-function new-alpha new-beta)))))
                              (cons move-eval
                                    (cond
                                      ((null? (cdr moves)) '()) ;Base case, last move was examined
                                      ((and max? (>= (cadr move-eval) new-beta)) '()) ;The move exceeds the bound, so we can ignore the rest
                                      ((and (not max?) (<= (cadr move-eval) new-alpha)) '()) 
                                      (else (examine (cdr moves)
                                                     (if max? (max new-alpha (cadr move-eval)) new-alpha) ;If we're minimizing, the alpha is the smaller of the past alpha and our current
                                                     (if max? new-beta (min new-beta (cadr move-eval)))   ;If we're maximizing, the beta is the larger of the past beta and our current
                                                     ))
                                    )
                              )
                            )
                     )))
             (examine (valid-moves position) alpha beta))
      )
 ) 
)

;(alpha-beta-search startpos 6 'W #t simple-evaluate -2000 2000)

;; Now you should be able to search twice as deep in the same amount of time.
;; (Of course, this alpha-beta-player won't work untl you've defined alpha-beta-search.)
;(play-game startpos (chatty-alpha-beta-player 10 focused-evaluate) (chatty-alpha-beta-player 10 focused-evaluate))

;; Finally, come up with a better evaluation function than focused-evaluate. By providing
;; a different function, you should be able to beat focused-evaluate while searching to the
;; same depth.

(define (better-evaluate pos player depth)
  (let ((simple-result (simple-evaluate pos player depth)) (opponent (if (eq? player 'W) 'B 'W)))
    (let (
    ;1) Break ties by favoring our turn (WIN RATIO 5/10) (NO IMPROVEMENT)
    ;(my-turn-advantage (if (eq? (get-color pos) player) .5 0))
    ;2) Favor having more pieces on the board (WIN RATIO 6/10) (seems to help with low-depth searches, but not with farsighted evaluations)
    ;(more-pieces-advantage (- (num-color player pos) (num-color opponent pos)))
    ;3) Favor having further pieces on the board (WIN RATIO 7/10) (*hindered* by the more-pieces-advantage heuristic)
    (furthest-piece-advantage (- (furthest-piece player pos) (furthest-piece opponent pos)))
    ;4) Favor having further pieces which are unblocked (WIN RATIO 5/10) (*worse* than simply furthest-piece advantage!)
    ;(furthest-unblocked-advantage (- (furthest-unblocked player pos) (furthest-unblocked player pos)))
    )
    (+ furthest-piece-advantage simple-result))
  )  
)

;Finds how many pieces on the board are of the matching color
(define (num-color player pos)
 (foldl + 0 (map (lambda (x) (if (eq? player (car x)) 1 0)) (get-board pos))) 
)

;Finds the player's piece which has travelled furthest
(define (furthest-piece player pos)
 (foldl max 0 (map 
               (lambda (x) 
                 (if (eq? player (car x)) 
                     (if (eq? 'W (car x)) (third x) (- 5 (third x)))
                     0
                 )
               )
               (get-board pos)
              )
 )
)
  
;Finds the player's piece which has travelled furthest and isn't blocked
(define (furthest-unblocked player pos)
 (furthest-piece player (list 'position (get-color pos) (remove-blocked (get-board pos) pos)))
)  

(define (remove-blocked pieces pos)
 (if (pair? pieces)
     (let ((piece (car pieces)))
       (if (eq? 'W (first piece)) 
           (if (and (is-piece? (second piece) (+ 1 (third piece)) 'B pos) 
                    (not (or (is-piece? (+ (second piece) 1) (+ 1 (third piece)) 'B pos) (is-piece? (- (second piece) 1) (+ 1 (third piece)) 'B pos))))
               (remove-blocked (cdr pieces) pos) 
               (cons piece (remove-blocked (cdr pieces) pos)))
           (if (and (is-piece? (second piece) (- (third piece) 1) 'W pos) 
                    (not (or (is-piece? (+ (second piece) 1) (- 1 (third piece)) 'W pos) (is-piece? (- (second piece) 1) (- 1 (third piece)) 'W pos))))
               (remove-blocked (cdr pieces) pos) 
               (cons piece (remove-blocked (cdr pieces) pos)))
       )
     )
     '()
 )
)
  
;Returns true if a piece of the specified color exists at those coordinates  
(define (is-piece? x y color pos)
  (let loop ((pieces (get-board pos)))
    (if (pair? pieces)
        (if (and (eqv? color (first (car pieces))) (= x (second (car pieces))) (= y (third (car pieces))))
            #t
            (loop (cdr pieces))
        )
        #f
    )
  )
)

;(furthest-unblocked 'W '(position B ((W 3 3) (W 4 3) (B 1 2) (B 2 4) (B 3 4) (B 4 4))))
  
;(furthest-piece 'B '(position B ((W 3 2) (W 4 3) (B 1 2) (B 2 4) (B 3 4) (B 4 4))))

;(better-evaluate '(position B ((W 3 3) (W 4 1) (B 1 4) (B 2 4) (B 3 4) (B 4 4))) 'B 0)

;(play-game startpos (chatty-alpha-beta-player 5 better-evaluate) (chatty-alpha-beta-player 5 focused-evaluate))
;(play-game startpos (chatty-alpha-beta-player 5 focused-evaluate) (chatty-alpha-beta-player 5 better-evaluate))

(define how-many-hours-this-pset-took "-")
(define what-i-found-interesting "-")
(define what-i-found-boring "-")
