;;;; nnet-train.scm
(require (lib "list.ss"))
(load "learn-utils.scm")
(load "backprop.scm")

(define (pretty-print x)
  (display x)
  (newline))

;; Samples in the backprop code are all numbers and the class is at the end.
;; This converts the data in nnet.data to work with our neural nets.
(define (convert-sample x)
  (list (data-point-features x)
	(list (if (eq? (data-point-class x) 'high)
		  1.0 0.0))))

;; Read in the data set
(read-training-data "nnet.data")

;; Normalize it
(set! *training-data* 
      (normalize-training-data *training-data*))

;; Convert them and randomize the order.
(define *samples*
  (random-reorder 
   (map convert-sample *training-data*)))

;; Classifies a datapoint using a trained net.  For use with (test...)
(define (neural-classify data-point)
  (let* ((converted (convert-sample data-point))
	 (out (first
	       (compute-final-outputs 
		(forward-propagate-output 
		 (first converted)
		 *weights*))))
	 (prediction (if (>= out 0.5) 'high 'normal)))
    (display* "The prediction is " prediction "(" out ")"
	      ".  Correct is " (data-point-class data-point)
	      "."
	      )
    prediction))

;; Initial weights should be small
(set! *initial-weight-magnitude* 0.001)

;; A reasonable choice of rate for this (small) problem.  
(set! *rate* 1.0)
(define *start-rate* 1.0)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Initializes a neural net to be trained for XOR
(define (initialize-xor learning-rate)
  (set! *rate* learning-rate)
  (set! *weights* (initialize-weights `(2 2 1)))
  (display "\nXOR net initialized"))

;; Trains the XOR neural net for a given number of epochs, or
;; until the average error is less than target-error
(define (train-xor epochs target-error)
  (train epochs *xor-samples* *weights* target-error))



;; Initializes a neural net with the data in nnet.data
(define (initialize-classifier-net n-hidden learning-rate)
  (set! *rate* learning-rate)
  (set! *weights* (initialize-weights  `(2 ,n-hidden 1)))
  (display "\nDefault neural net initialized"))

;; Trains the neural net for a given number of epochs, or
;; until the average error is less than target-error
(define (train-classifier-net epochs target-error)
  (train epochs *samples* *weights* target-error))

;; Validate the neural net with classification tests
(define (validate-classifier-net)
  (test *training-data* neural-classify))


(initialize-classifier-net 4 .5)
(let input-loop ((continue 1) (n 1))
  (train-classifier-net 1000 .1)
  (display n)
  (if (= continue 1) (input-loop (read) (+ n 1)))
)
  
(validate-classifier-net)

;(initialize-classifier-net 1 1)
;(train-classifier-net 1000 .1)
;(validate-classifier-net)