(require (lib "list.ss"))

;; A list of the issues that were voted on
(define congress-attribute-names
  '("Handicapped Infants"
    "Water Project Cost Sharing"
    "Adopt the Budget Resolution"
    "Physician Fee Freeze"
    "Aid to El Salvador"
    "Religious Groups in Schools"
    "Anti Satellite Test Ban"
    "Aid to Nicaraguan Contras"
    "MX Missile"
    "Immigration"
    "Synfuels Corporation Cutback"
    "Education Spending"
    "Superfund Right to Sue"
    "Crime"
    "Duty Free Exports"
    "South Africa Export Administration Act"
    )
  )

;; Creates attribute-specs for the congressional issues
(define congress-attribute-specs
   (let loop ([cur 0])
     (if (= cur 16)
	 '()
	 (cons (list 
		(list-ref congress-attribute-names cur)
		(lambda (congressman) (list-ref congressman cur))
		'(yes no maybe)
                cur)
	       (loop (+ cur 1))))))

(define attribute-name car)
(define attribute-extractor cadr)
(define attribute-values caddr)
(define attribute-id cadddr)

(define (number-to-party n)
  (if (> n 0) "republican" "democrat"))

(define (party-to-number party)
  (if (equal? party "republican") 1 -1))

(define (convert-party-data data)
  (map (lambda (datum)
         (cons 1 (cons (car datum) (party-to-number (cdr datum)))))
       data))

(define get-weight car)
(define get-info cadr)
(define get-class cddr)
(define (make-datum weight info value)
  (cons weight (cons info value)))

;; Congressional data:
 
;; The first 25 entries of train-data-large
(define train-data-small
  (convert-party-data
  '(((no no no yes yes yes no no no yes yes yes yes yes no maybe) . "republican") 
    ((no no yes no no yes yes yes no no yes no no yes maybe yes) . "democrat") 
    ((yes no yes no no no yes yes yes no no no no no yes yes) . "democrat") 
    ((yes no yes no no no yes yes yes yes no no no yes yes yes) . "democrat") 
    ((yes no no yes yes yes no no no no yes yes yes yes no no) . "republican") 
    ((no maybe yes maybe no yes yes yes yes yes yes no maybe maybe yes yes) . "democrat") 
    ((no yes yes no yes maybe yes no no yes yes no yes no yes yes) . "democrat") 
    ((no no no yes yes no yes no yes yes no no no yes no yes) . "republican") 
    ((no no yes no no no yes yes yes yes yes no no no yes yes) . "democrat")
    ((no no no yes yes yes yes no no yes no yes no yes yes yes) . "republican") 
    ((no no no yes yes yes no no no yes no yes yes yes no yes) . "republican") 
    ((yes no no yes yes yes no no no yes no yes yes yes no no) . "republican")
    ((yes no yes no no no yes yes yes yes no yes no no yes maybe) . "democrat")
    ((no yes yes yes yes yes yes yes yes no no yes yes yes no yes) . "republican")
    ((no yes no no no yes yes no yes no yes no no no yes yes) . "democrat") 
    ((no no yes yes yes yes yes yes yes yes no yes yes yes yes yes) . "republican") 
    ((no yes no yes no yes yes yes yes no yes no yes no yes maybe) . "democrat") 
    ((no no yes yes yes yes yes no no yes yes yes yes yes no yes) . "republican")
    ((no yes yes no no yes yes yes yes yes no maybe no no yes yes) . "democrat") 
    ((yes no yes yes no no no yes yes yes no no no yes yes yes) . "republican") 
    ((no no no yes yes yes no no no no no yes yes yes no no) . "republican")
    ((no no no yes yes yes no no no no no yes yes yes no no) . "republican") 
    ((yes yes yes no no yes yes yes yes yes yes yes yes yes no maybe) . "democrat") 
    ((no no no yes yes yes no no no yes maybe yes yes yes no yes) . "republican")
    ((yes no yes no no yes yes yes yes yes no no yes no no yes) . "democrat"))))

(define train-data-large 
  (convert-party-data
  '(((no no no yes yes yes no no no yes yes yes yes yes no maybe) . "republican") 
    ((no no yes no no yes yes yes no no yes no no yes maybe yes) . "democrat") 
    ((yes no yes no no no yes yes yes no no no no no yes yes) . "democrat") 
    ((yes no yes no no no yes yes yes yes no no no yes yes yes) . "democrat") 
    ((yes no no yes yes yes no no no no yes yes yes yes no no) . "republican") 
    ((no maybe yes maybe no yes yes yes yes yes yes no maybe maybe yes yes) . "democrat") 
    ((no yes yes no yes maybe yes no no yes yes no yes no yes yes) . "democrat") 
    ((no no no yes yes no yes no yes yes no no no yes no yes) . "republican") 
    ((no no yes no no no yes yes yes yes yes no no no yes yes) . "democrat")
    ((no no no yes yes yes yes no no yes no yes no yes yes yes) . "republican") 
    ((no no no yes yes yes no no no yes no yes yes yes no yes) . "republican") 
    ((yes no no yes yes yes no no no yes no yes yes yes no no) . "republican")
    ((yes no yes no no no yes yes yes yes no yes no no yes maybe) . "democrat")
    ((no yes yes yes yes yes yes yes yes no no yes yes yes no yes) . "republican")
    ((no yes no no no yes yes no yes no yes no no no yes yes) . "democrat") 
    ((no no yes yes yes yes yes yes yes yes no yes yes yes yes yes) . "republican") 
    ((no yes no yes no yes yes yes yes no yes no yes no yes maybe) . "democrat") 
    ((no no yes yes yes yes yes no no yes yes yes yes yes no yes) . "republican")
    ((no yes yes no no yes yes yes yes yes no maybe no no yes yes) . "democrat") 
    ((yes no yes yes no no no yes yes yes no no no yes yes yes) . "republican") 
    ((no no no yes yes yes no no no no no yes yes yes no no) . "republican")
    ((no no no yes yes yes no no no no no yes yes yes no no) . "republican") 
    ((yes yes yes no no yes yes yes yes yes yes yes yes yes no maybe) . "democrat") 
    ((no no no yes yes yes no no no yes maybe yes yes yes no yes) . "republican")
    ((yes no yes no no yes yes yes yes yes no no yes no no yes) . "democrat")
    ((yes no yes no yes yes yes no yes yes no no yes yes no maybe) . "democrat")
    ((yes yes yes no no yes yes yes yes yes yes yes yes no no yes) . "democrat")
    ((yes yes no yes yes yes no no no yes yes no yes no no no) . "republican") 
    ((yes yes no yes yes yes no no no no yes no yes yes no yes) . "republican")
    ((no yes no no yes yes no no no yes yes no yes yes no no) . "democrat") 
    ((yes no yes no no no yes yes no yes yes no no no no maybe) . "democrat") 
    ((yes yes yes no yes yes yes yes no yes yes no no no yes maybe) . "democrat")
    ((no yes yes no no yes yes yes no yes no no no no yes yes) . "democrat") 
    ((no yes no yes yes yes no no no no no no yes yes no yes) . "republican")
    ((yes yes yes no maybe yes yes yes no yes maybe maybe no no yes yes) . "democrat")
    ((yes yes yes no maybe no yes yes yes yes no no no no yes maybe) . "democrat") 
    ((no yes yes yes yes yes no no no no yes yes maybe yes no no) . "democrat") 
    ((no yes yes maybe yes yes no yes no yes maybe no yes yes maybe yes) . "democrat") 
    ((no yes no yes yes yes no no no no no yes yes yes no yes) . "republican") 
    ((no yes no yes yes yes no no no no yes yes no yes no no) . "democrat") 
    ((yes maybe yes no no no yes yes yes no yes no no no yes yes) . "democrat") 
    ((no yes no yes yes yes maybe maybe no no maybe maybe yes maybe maybe maybe) . "republican")
    ((no no no yes yes yes no no no no no yes yes yes no yes) . "republican") 
    ((no no no yes yes yes no no no no no yes yes yes no yes) . "republican") 
    ((yes yes yes no no yes maybe yes yes no yes no yes no yes yes) . "democrat") 
    ((yes yes yes no yes yes yes yes yes yes yes no yes yes no maybe) . "democrat")
    ((yes yes no yes yes yes no no no no yes no yes yes no maybe) . "democrat")
    ((yes yes yes no yes yes no yes yes yes yes no no no no yes) . "democrat") 
    ((yes yes yes yes yes yes no no no no yes yes yes yes no yes) . "democrat")
    ((yes yes no no yes yes no no no no yes yes yes yes yes no) . "democrat") 
    ((no maybe yes no yes yes no yes no no yes no no no no maybe) . "democrat") 
    ((yes yes yes no yes yes no yes yes no yes no no yes no maybe) . "democrat")
    ((no yes yes yes yes yes no no no no no yes yes yes no maybe) . "democrat") 
    ((yes no yes no no no yes yes yes maybe yes no no no yes maybe) . "democrat")
    ((maybe maybe no no maybe yes maybe no no no yes yes no yes no maybe) . "democrat") 
    ((yes yes no no no no no yes yes no yes no no no yes no) . "democrat") 
    ((yes yes no yes yes yes no no no no yes yes yes yes no yes) . "republican") 
    ((maybe maybe maybe maybe no yes no yes yes no no yes yes no no maybe) . "republican") 
    ((yes yes maybe maybe maybe yes no no no no yes no yes no no yes) . "democrat")
    ((yes yes yes maybe no no no yes no no yes maybe no no yes yes) . "democrat") 
    ((yes yes yes no yes yes no yes no no yes no yes no yes yes) . "democrat") 
    ((yes yes no no yes maybe no no no no yes no yes yes no yes) . "democrat") 
    ((no yes yes no yes yes no yes no no no no no no no yes) . "democrat") 
    ((no yes no yes maybe yes no no no yes no yes yes yes no no) . "republican")
    ((no yes no yes yes yes no maybe no no maybe maybe maybe yes no maybe) . "republican")
    ((no yes no yes yes yes no no no yes yes yes yes yes no no) . "republican")
    ((maybe no yes yes no yes yes yes yes yes no yes no yes no yes) . "republican") 
    ((no yes no yes yes yes no no no yes no yes maybe yes no no) . "republican") 
    ((yes yes no yes yes yes no no no yes no yes yes yes no yes) . "republican")
    ((no no no yes yes yes no no no no no yes yes yes no yes) . "republican") 
    ((yes no yes no yes yes no no yes yes no no yes yes no yes) . "democrat") 
    ((no no no yes yes yes no no no no yes yes yes yes no no) . "democrat") 
    ((yes no yes no no yes yes yes yes no no yes maybe yes yes yes) . "democrat") 
    ((no no no yes yes yes no no no no no yes yes yes no no) . "republican") 
    ((no no no yes yes yes no no no no yes yes yes yes no yes) . "republican")
    ((yes no yes no no yes yes yes yes yes yes no no no no yes) . "democrat") 
    ((no no no yes yes yes no no no yes no yes yes yes no yes) . "republican") 
    ((yes yes yes yes yes yes yes yes no yes maybe maybe maybe yes no yes) . "republican") 
    ((yes yes yes no no no yes yes yes no no no no no no yes) . "democrat")
    ((no yes yes no no yes yes yes maybe yes no no no no no yes) . "democrat") 
    ((yes yes no yes yes yes no no no yes no no yes yes no yes) . "republican")
    ((yes yes yes no no no yes yes yes yes yes no yes no no yes) . "democrat") 
    ((yes yes yes no no no yes yes no yes no no no no no yes) . "democrat") 
    ((yes yes yes no no no yes yes yes no no no no no no yes) . "democrat") 
    ((yes yes yes yes yes yes yes yes no yes no no yes yes no yes) . "republican")
    ((no yes yes no yes yes yes yes no no yes no yes no yes yes) . "democrat")
    ((no no yes no no yes yes yes yes no yes no no no yes yes) . "democrat")
    ((no yes yes no no yes yes yes yes no yes no no yes yes yes) . "democrat") 
    ((no yes yes no no maybe yes yes yes yes yes no maybe yes yes yes) . "democrat") 
    ((no no yes no no no yes yes no yes yes no no no yes maybe) . "democrat") 
    ((yes no yes no no no yes yes yes yes no no no no yes yes) . "democrat") 
    ((no no no yes yes yes yes yes no yes no yes yes yes no yes) . "republican")
    ((maybe maybe maybe no no no yes yes yes yes no no yes no yes yes) . "democrat")
    ((yes no yes no maybe no yes yes yes yes no yes no maybe yes yes) . "democrat") 
    ((no no yes yes yes yes no no yes yes no yes yes yes no yes) . "republican") 
    ((no no yes no no no yes yes yes yes no no no no no yes) . "democrat") 
    ((no maybe no yes yes yes no no no no yes yes yes yes no yes) . "republican") 
    ((no no no yes yes yes maybe maybe maybe maybe no yes yes yes no yes) . "republican")
    ((no yes no yes yes yes no no no yes no yes yes yes maybe no) . "republican"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; The first 25 entries of test-data-large
(define test-data-small
  (convert-party-data
  '(((no no no yes yes yes no no no yes no yes no yes no yes) . "republican")
    ((no no yes no no no yes yes yes no no no no no yes yes) . "democrat")
    ((yes no yes no no yes yes yes no no no yes yes no no yes) . "democrat")
    ((yes yes yes no no no yes yes maybe yes no no no no yes maybe) . "democrat")
    ((no no no yes yes yes yes no no yes no no no yes yes yes) . "republican")
    ((no no no yes no yes yes maybe yes no no yes yes yes no yes) . "republican")
    ((yes no yes no no no yes yes yes yes yes no no yes yes yes) . "democrat")
    ((no no no no yes yes yes no no no no maybe no yes yes yes) . "republican")
    ((no yes yes no no no yes yes maybe yes no no yes no yes yes) . "democrat")
    ((yes no yes no no no no yes yes yes no no no no yes yes) . "democrat")
    ((yes no yes no no no yes yes yes yes yes no no no yes yes) . "democrat")
    ((no no yes no yes no yes yes yes no no no no yes maybe yes) . "democrat")
    ((no yes no yes yes yes maybe no no no no maybe yes yes no no) . "republican")
    ((maybe maybe maybe maybe maybe maybe maybe maybe maybe maybe maybe maybe maybe maybe maybe maybe) . "republican")
    ((yes no yes no no no yes yes maybe no yes no no no yes yes) . "democrat")
    ((no yes no yes yes yes no no no no no yes yes yes no no) . "republican")
    ((no yes no yes yes yes no no no no no yes yes yes no no) . "republican")
    ((yes yes yes no no yes yes yes yes no no no no no yes yes) . "democrat")
    ((no yes no yes yes yes no no no no no yes yes yes no yes) . "republican")
    ((yes no yes no no no yes yes yes yes no no no no no yes) . "democrat")
    ((yes no yes no no no yes yes yes yes no no no yes yes yes) . "democrat")
    ((no no no yes yes no no no no no no yes no yes no no) . "republican")
    ((no no no yes yes no no no no no no yes no yes maybe yes) . "republican")
    ((no no yes no no no yes yes yes no yes no no no yes yes) . "democrat")
    ((yes no yes no no no yes yes yes no no no no no no yes) . "democrat"))))

(define test-data-large
  (convert-party-data
  '(((no no no yes yes yes no no no yes no yes no yes no yes) . "republican")
    ((no no yes no no no yes yes yes no no no no no yes yes) . "democrat")
    ((yes no yes no no yes yes yes no no no yes yes no no yes) . "democrat")
    ((yes yes yes no no no yes yes maybe yes no no no no yes maybe) . "democrat")
    ((no no no yes yes yes yes no no yes no no no yes yes yes) . "republican")
    ((no no no yes no yes yes maybe yes no no yes yes yes no yes) . "republican")
    ((yes no yes no no no yes yes yes yes yes no no yes yes yes) . "democrat")
    ((no no no no yes yes yes no no no no maybe no yes yes yes) . "republican")
    ((no yes yes no no no yes yes maybe yes no no yes no yes yes) . "democrat")
    ((yes no yes no no no no yes yes yes no no no no yes yes) . "democrat")
    ((yes no yes no no no yes yes yes yes yes no no no yes yes) . "democrat")
    ((no no yes no yes no yes yes yes no no no no yes maybe yes) . "democrat")
    ((no yes no yes yes yes maybe no no no no maybe yes yes no no) . "republican")
    ((maybe maybe maybe maybe maybe maybe maybe maybe maybe maybe maybe maybe maybe maybe maybe maybe) . "republican")
    ((yes no yes no no no yes yes maybe no yes no no no yes yes) . "democrat")
    ((no yes no yes yes yes no no no no no yes yes yes no no) . "republican")
    ((no yes no yes yes yes no no no no no yes yes yes no no) . "republican")
    ((yes yes yes no no yes yes yes yes no no no no no yes yes) . "democrat")
    ((no yes no yes yes yes no no no no no yes yes yes no yes) . "republican")
    ((yes no yes no no no yes yes yes yes no no no no no yes) . "democrat")
    ((yes no yes no no no yes yes yes yes no no no yes yes yes) . "democrat")
    ((no no no yes yes no no no no no no yes no yes no no) . "republican")
    ((no no no yes yes no no no no no no yes no yes maybe yes) . "republican")
    ((no no yes no no no yes yes yes no yes no no no yes yes) . "democrat")
    ((yes no yes no no no yes yes yes no no no no no no yes) . "democrat")
    ((yes no yes no no no yes yes yes yes no no no no no yes) . "democrat")
    ((yes no yes no no maybe yes yes yes no maybe maybe no maybe maybe maybe) . "democrat")
    ((yes no yes no no no yes yes yes yes no no maybe no yes yes) . "democrat")
    ((yes no yes no no no yes yes yes no no no no no yes maybe) . "democrat")
    ((yes no yes no no no yes yes yes no no no no no yes maybe) . "democrat")
    ((yes no yes no no no yes yes yes yes no no no no no yes) . "democrat")
    ((no no no yes yes yes no no no yes no yes no yes no yes) . "republican")
    ((yes no no no no no yes yes yes yes no no no yes no yes) . "republican")
    ((yes no yes no no no yes yes yes no no no no no yes maybe) . "democrat")
    ((yes no yes no no no yes yes yes no no no no no no yes) . "democrat")
    ((yes yes yes no no no yes yes yes no no no no no yes yes) . "democrat")
    ((no yes yes no no yes yes yes yes no maybe no no no no yes) . "democrat")
    ((yes no yes no no no yes yes yes yes no no no no yes maybe) . "democrat")
    ((no no no yes yes no yes yes no yes no yes yes yes maybe yes) . "republican")
    ((yes no no yes yes no yes no no yes no no no yes yes yes) . "republican")
    ((no no yes no yes yes no no no no maybe no yes yes no no) . "democrat")
    ((no no no yes yes yes no no no no no yes yes yes yes no) . "republican")
    ((no no yes yes yes yes yes yes no yes no no no yes no yes) . "republican")
    ((no no no yes yes yes no no no no no yes yes yes no yes) . "republican")
    ((no no no yes yes yes no no no yes no yes yes yes no no) . "republican")
    ((no no yes no no no yes yes yes yes no no no yes no yes) . "democrat")
    ((yes no yes yes yes yes yes yes no no no no no yes no maybe) . "republican")
    ((yes no no yes yes yes no no no yes no maybe yes yes no no) . "republican")
    ((no no no yes yes yes no no no no no yes yes yes no yes) . "republican")
    ((no no yes no no yes yes yes yes yes yes no no no maybe yes) . "democrat")
    ((no no yes no no yes yes yes yes yes yes no no no yes yes) . "democrat")
    ((no no yes no no yes maybe yes maybe yes yes yes no yes yes maybe) . "democrat")
    ((yes yes yes maybe no yes yes yes yes no yes no yes no maybe yes) . "democrat")
    ((yes yes yes no yes yes no yes no yes yes no yes yes yes yes) . "democrat")
    ((yes yes yes no yes yes no yes no yes yes no yes yes no maybe) . "democrat")
    ((yes no yes no maybe yes maybe yes yes yes no no yes yes no yes) . "democrat")
    ((yes no yes no no yes yes yes yes yes no maybe no yes no yes) . "democrat")
    ((yes no yes no no yes yes yes no yes yes no yes yes yes yes) . "democrat")
    ((yes yes yes no no yes yes yes yes yes yes no yes yes yes yes) . "democrat")
    ((no yes yes no no yes yes yes no yes yes no yes yes no maybe) . "democrat")
    ((no yes no yes yes yes maybe maybe no yes no yes maybe maybe maybe maybe) . "republican")
    ((no no yes yes yes yes no no no yes no yes yes yes yes yes) . "republican")
    ((yes yes yes no no yes yes yes yes yes no no maybe no yes maybe) . "democrat")
    ((no yes no no no no yes yes yes yes yes no no no yes yes) . "democrat")
    ((no yes yes no no yes yes yes yes yes no no yes yes yes yes) . "democrat")
    ((no no no yes yes no yes yes yes yes no yes yes yes no yes) . "republican")
    ((no no maybe no no yes yes yes yes no no no no no yes yes) . "democrat")
    ((no no no yes yes yes yes no no yes no yes yes yes no yes) . "republican")
    ((no no no yes yes yes no no no no no yes yes yes no no) . "republican")
    ((no yes no yes yes yes no no no yes no yes yes yes no maybe) . "republican")
    ((no no no yes yes yes no no no yes no yes yes yes no no) . "republican")
    ((no no no yes yes yes no no no no no yes yes yes no no) . "republican")
    ((yes no yes no no yes yes yes yes no no no no yes no maybe) . "democrat")
    ((no no no yes yes yes no no no yes no yes yes yes no no) . "republican")
    ((yes no no no no yes yes yes yes yes no no no yes yes yes) . "democrat")
    ((no no no yes yes yes no no no yes no yes yes yes yes no) . "republican")
    ((no no yes no no yes yes yes yes yes no no yes no no yes) . "democrat")
    ((yes yes yes no no no yes yes yes yes no no no no yes yes) . "democrat")
    ((no yes yes yes yes yes no no no yes no yes yes yes no yes) . "republican")
    ((no yes no yes yes yes yes yes no no yes yes yes yes yes yes) . "republican")
    ((no yes yes yes yes yes yes maybe no no no no maybe maybe yes maybe) . "republican")
    ((no no no no no yes no yes yes no yes yes yes yes yes no) . "democrat")
    ((yes no no no no no yes yes yes yes no no no no yes yes) . "democrat")
    ((no no yes no no no yes yes yes no no no no no yes maybe) . "democrat")
    ((yes no yes no no no yes yes yes no no no no no yes maybe) . "democrat")
    ((no yes yes no no yes no yes yes yes no no yes yes no yes) . "democrat")
    ((yes yes yes no no no yes yes yes yes no no yes no no yes) . "democrat")
    ((yes yes yes no maybe yes no maybe no no yes no yes yes no maybe) . "democrat")
    ((yes yes yes no yes yes no yes maybe yes no no yes yes no maybe) . "democrat")
    ((no yes no yes yes yes no no no no yes yes yes yes no no) . "republican")
    ((no yes no no yes yes no no maybe no no yes yes yes no yes) . "democrat")
    ((yes yes no yes no no yes yes yes no yes no no yes no yes) . "democrat")
    ((no yes no yes yes yes no no no no no yes yes yes no yes) . "republican")
    ((yes yes yes no no no yes yes yes no yes no no no no yes) . "democrat")
    ((yes maybe yes no no yes yes yes yes yes no no no no yes maybe) . "democrat")
    ((no yes no yes yes yes no no no yes no yes yes yes no no) . "republican")
    ((yes maybe yes no no no yes yes yes no no no no no yes maybe) . "democrat")
    ((yes no yes no no no yes yes yes no yes no no no yes maybe) . "democrat")
    ((no no yes no no no yes yes yes no no no no no yes yes) . "democrat")
    ((no yes yes no no yes yes yes maybe no yes yes no no yes yes) . "democrat"))))
